from flask import abort, request
from app import db
from app.decorators import content_negotiation
from app.models import (
    Corpus,
    CorpusFollowerAssociation,
    CorpusFollowerRole,
    User
)
from ..decorators import corpus_follower_permission_required
from . import bp


@bp.route('/<hashid:corpus_id>/followers', methods=['POST'])
@corpus_follower_permission_required('ADD_FOLLOWER')
@content_negotiation(consumes='application/json', produces='application/json')
def create_corpus_followers(corpus_id):
    usernames = request.json
    if not (isinstance(usernames, list) or all(isinstance(u, str) for u in usernames)):
        abort(400)
    corpus = Corpus.query.get_or_404(corpus_id)
    for username in usernames:
        user = User.query.filter_by(username=username, is_public=True).first_or_404()
        user.follow_corpus(corpus)
    db.session.commit()
    response_data = {
        'message': f'Users are now following "{corpus.title}"',
        'category': 'corpus'
    }
    return response_data, 200


@bp.route('/<hashid:corpus_id>/followers/<hashid:follower_id>/role', methods=['PUT'])
@corpus_follower_permission_required('UPDATE_FOLLOWER')
@content_negotiation(consumes='application/json', produces='application/json')
def update_corpus_follower_role(corpus_id, follower_id):
    role_name = request.json
    if not isinstance(role_name, str):
        abort(400)
    cfr = CorpusFollowerRole.query.filter_by(name=role_name).first()
    if cfr is None:
        abort(400)
    cfa = CorpusFollowerAssociation.query.filter_by(corpus_id=corpus_id, follower_id=follower_id).first_or_404()
    cfa.role = cfr
    db.session.commit()
    response_data = {
        'message': f'User "{cfa.follower.username}" is now {cfa.role.name}',
        'category': 'corpus'
    }
    return response_data, 200


@bp.route('/<hashid:corpus_id>/followers/<hashid:follower_id>', methods=['DELETE'])
@corpus_follower_permission_required('REMOVE_FOLLOWER')
@content_negotiation(produces='application/json')
def delete_corpus_follower(corpus_id, follower_id):
    cfa = CorpusFollowerAssociation.query.filter_by(corpus_id=corpus_id, follower_id=follower_id).first_or_404()
    cfa.follower.unfollow_corpus(cfa.corpus)
    db.session.commit()
    response_data = {
        'message': (
            f'"{cfa.follower.username}" is not following '
            f'"{cfa.corpus.title}" anymore'
        ),
        'category': 'corpus'
    }
    return response_data, 200
